@@ -528,6 +528,17 @@ def _generate_metadata_contributor_xml(contributors: List[Dict], contributor_typ
 
 	return output.strip()
 
+def _generate_titlepage_string(contributors: List[Dict], contributor_type: str) -> str:
+	output = _generate_contributor_string(contributors, True)
+	output = regex.sub(r"<a href[^<>]+?>", "<b epub:type=\"z3998:personal-name\">", output)
+	output = output.replace("</a>", "</b>")
+
+	if contributor_type != "illustrator":
+		# There's no z3998:illustrator term
+		output = output.replace("\"z3998:personal-name", f"\"z3998:{contributor_type} z3998:personal-name")
+
+	return output
+
 def _create_draft(args: Namespace):
 	"""
 	Implementation for `se create-draft`
@@ -772,13 +783,28 @@ def _create_draft(args: Namespace):
 	if not args.offline and title not in ("Short Fiction", "Poetry", "Essays", "Plays"):
 		ebook_wiki_url, _ = _get_wikipedia_url(title, False)
 
-	if args.white_label:
-		_replace_in_file(content_path / "epub" / "text" / "titlepage.xhtml", "TITLE", escape(title))
-		_replace_in_file(content_path / "epub" / "text" / "titlepage.xhtml", "AUTHOR", escape(_generate_contributor_string(authors, False)))
+	with open(content_path / "epub" / "text" / "titlepage.xhtml", "r+", encoding="utf-8") as file:
+		titlepage_xhtml = file.read()
 
-	else:
-		_replace_in_file(content_path / "epub" / "text" / "titlepage.xhtml", "TITLE_STRING", escape(title_string))
+		titlepage_xhtml = titlepage_xhtml.replace("TITLE", escape(title))
 
+		titlepage_xhtml = titlepage_xhtml.replace("AUTHOR", _generate_titlepage_string(authors, "author"))
+
+		if translators:
+			titlepage_xhtml = titlepage_xhtml.replace("TRANSLATOR", _generate_titlepage_string(translators, "translator"))
+		else:
+			titlepage_xhtml = regex.sub(r"<p>Translated by.+?</p>", "", titlepage_xhtml, flags=regex.DOTALL)
+
+		if illustrators:
+			titlepage_xhtml = titlepage_xhtml.replace("ILLUSTRATOR", _generate_titlepage_string(illustrators, "illustrator"))
+		else:
+			titlepage_xhtml = regex.sub(r"<p>Illustrated by.+?</p>", "", titlepage_xhtml, flags=regex.DOTALL)
+
+		file.seek(0)
+		file.write(se.formatting.format_xhtml(titlepage_xhtml))
+		file.truncate()
+
+	if not args.white_label:
 		# Create the titlepage SVG
 		contributors = {}
 		if translators:
