@@ -349,7 +349,7 @@ def get_word_count(xhtml: str) -> int:
 	xhtml = regex.sub(r"<(m:)?math.+?</(m:)?math>", " ", xhtml)
 
 	# Remove HTML tags
-	xhtml = regex.sub(r"<title>.+?</title>", " ", xhtml)
+	xhtml = regex.sub(r"<title[^>]*?>.+?</title>", " ", xhtml)
 	xhtml = regex.sub(r"<.+?>", " ", xhtml, flags=regex.DOTALL)
 
 	# Replace some formatting characters
@@ -791,7 +791,10 @@ def _format_css_component_list(content: list, in_selector=False, in_paren_block=
 			if output.endswith("| ") or output.endswith("|= ") or output.endswith("^= ") or output.endswith("~= ") or output.endswith("*= ") or output.endswith("|| "):
 				output = output.rstrip()
 
-			output += token.lower_value
+			if token.lower_value == "currentcolor": # special case
+				output += "currentColor"
+			else:
+				output += token.lower_value
 
 		if token.type == "literal":
 			if token.value == ":" and in_paren_block:
@@ -1230,9 +1233,9 @@ def namespace_to_class(selector: str) -> str:
 	"""
 
 	# First, remove periods from epub:type.	 We can't remove periods in the entire selector because there might be class selectors involved
-	epub_type = regex.search(r"\"[^\"]+?\"", selector).group()
+	epub_type = regex.search(r"\"[^\"]+?\"", selector)
 	if epub_type:
-		selector = selector.replace(epub_type, epub_type.replace(".", "-"))
+		selector = selector.replace(epub_type.group(), epub_type.group().replace(".", "-"))
 
 	# Now clean things up
 	return selector.replace(":", "-").replace("|", "-").replace("~=", "-").replace("[", ".").replace("]", "").replace("\"", "")
@@ -1284,7 +1287,7 @@ def simplify_css(css: str) -> str:
 
 	# Replace CSS namespace selectors with classes
 	# For example, p[epub|type~="z3998:salutation"] becomes p.epub-type-z3998-salutation
-	for line in regex.findall(r"\[epub\|type\~\=\"[^\"]*?\"\]", css):
+	for line in regex.findall(r"\[[a-z]+\|[a-z]+(?:\~\=\"[^\"]*?\")?\]", css):
 		fixed_line = namespace_to_class(line)
 		css = css.replace(line, fixed_line)
 
@@ -1310,6 +1313,10 @@ def generate_title(xhtml: Union[str, EasyXmlTree]) -> str:
 	except Exception as ex:
 		raise se.InvalidXhtmlException(f"Couldn’t parse XHTML file. Exception: {ex}")
 
+	# Titlepages are the exception
+	if dom.xpath("/html/body//section[re:test(@epub:type, '\\btitlepage\\b')]"):
+		return "Titlepage"
+
 	title = ""
 
 	# Do we have an hgroup element to process?
