@@ -364,8 +364,11 @@ class SeEpub:
 		file_path_str = str(file_path)
 
 		if file_path_str not in self._file_cache:
-			with open(file_path, "r", encoding="utf-8") as file:
-				file_contents = file.read()
+			try:
+				with open(file_path, "r", encoding="utf-8") as file:
+					file_contents = file.read()
+			except Exception as ex:
+				raise se.InvalidFileException(f"Couldn’t read file: [path]{file_path_str}[/]") from ex
 
 			self._file_cache[file_path_str] = file_contents
 
@@ -432,7 +435,6 @@ class SeEpub:
 	def _recompose_xhtml(self, section: se.easy_xml.EasyXmlElement, output_dom: se.easy_xml.EasyXmlTree) -> None:
 		"""
 		Helper function used in self.recompose()
-		Recursive function for recomposing a series of XHTML files into a single XHTML file.
 
 		INPUTS
 		section: An EasyXmlElement to inspect
@@ -444,21 +446,20 @@ class SeEpub:
 
 		# Quick sanity check before we begin
 		if not section.get_attr("id") or (section.parent.tag.lower() != "body" and not section.parent.get_attr("id")):
-			raise se.InvalidXhtmlException("Section without [attr]id[/] attribute.")
+			raise se.InvalidXhtmlException(f"Section without [attr]id[/] attribute: [html]{section.to_tag_string()}[/]")
 
-		if section.parent.tag.lower() == "body":
+		if section.parent.tag.lower() == "body" and not section.get_attr("data-parent"):
 			section.set_attr("epub:type", f"{section.get_attr('epub:type')} {section.parent.get_attr('epub:type')}".strip())
 
-		# Try to find our parent tag in the output, by ID.
-		# If it's not in the output, then append it to the tag's closest parent by ID (or <body>), then iterate over its children and do the same.
-		existing_section = output_dom.xpath(f"//*[@id='{section.get_attr('id')}']")
-		if not existing_section:
-			if section.parent.tag.lower() == "body":
-				output_dom.xpath("/html/body")[0].append(section)
-			else:
-				output_dom.xpath(f"//*[@id='{section.parent.get_attr('id')}']")[0].append(section)
+		# Try to find our parent element in the current output dom, by ID.
+		# If it's not in the output, then append this element to the elements's closest parent by ID (or <body>), then iterate over its children and do the same.
+		existing_section = None
+		existing_section = output_dom.xpath(f"//*[@id='{section.get_attr('data-parent')}']")
 
-			existing_section = output_dom.xpath(f"//*[@id='{section.get_attr('id')}']")
+		if existing_section:
+			existing_section[0].append(section)
+		else:
+			output_dom.xpath("/html/body")[0].append(section)
 
 		# Convert all <img> references to inline base64
 		# We even convert SVGs instead of inlining them, because CSS won't allow us to style inlined SVGs
@@ -469,19 +470,13 @@ class SeEpub:
 				image_contents_base64 = base64.b64encode(binary_file.read()).decode()
 
 			if src.endswith(".svg"):
-				img.set_attr("src", f"data:image/svg+xml;base64, {image_contents_base64}")
+				img.set_attr("src", f"data:image/svg+xml;base64,{image_contents_base64}")
 
 			if src.endswith(".jpg"):
-				img.set_attr("src", f"data:image/jpg;base64, {image_contents_base64}")
+				img.set_attr("src", f"data:image/jpg;base64,{image_contents_base64}")
 
 			if src.endswith(".png"):
-				img.set_attr("src", f"data:image/png;base64, {image_contents_base64}")
-
-		for child in section.xpath("./*"):
-			if child.tag in ("section", "article"):
-				self._recompose_xhtml(child, output_dom)
-			else:
-				existing_section.append(child)
+				img.set_attr("src", f"data:image/png;base64,{image_contents_base64}")
 
 	def recompose(self, output_xhtml5: bool, extra_css_file: Path = None) -> str:
 		"""
@@ -517,7 +512,7 @@ class SeEpub:
 
 		css = css.strip()
 
-		namespaces = list(set(namespaces))
+		namespaces = sorted(list(set(namespaces)), reverse=True)
 
 		if namespaces:
 			css = "\n" + css
@@ -564,6 +559,10 @@ class SeEpub:
 				except se.SeException as ex:
 					raise se.SeException(f"[path][link=file://{file_path}]{file_path}[/][/]: {ex}") from ex
 
+		# Remove data-parent attributes
+		for node in output_dom.xpath("//*[@data-parent]"):
+			node.remove_attr("data-parent")
+
 		# Did we add wrappers? If so add the CSS
 		# We also have to give the wrapper a height, because it may have siblings that were recomposed in from other files
 		if needs_wrapper_css:
@@ -641,7 +640,13 @@ class SeEpub:
 
 			# Make some replacements for HTML5 compatibility
 			output_xhtml = output_xhtml.replace("epub|type", "data-epub-type")
+			output_xhtml = output_xhtml.replace("xml|lang", "lang")
 			output_xhtml = regex.sub(r" xmlns.+?=\".+?\"", "", output_xhtml)
+			output_xhtml = regex.sub(r"@namespace (epub|xml).+?\s+", "", output_xhtml, flags=regex.MULTILINE)
+
+			# The Nu HTML5 Validator barfs if non-void elements are self-closed (like <td/>)
+			# Try to un-self-close them for HTML5 output.
+			output_xhtml = regex.sub(r"<(colgroup|td|th|span)( [^/>]*?)?/>", r"<\1\2></\1>", output_xhtml)
 
 		return output_xhtml
 
