@@ -14,6 +14,7 @@ import tempfile
 from distutils.dir_util import copy_tree
 from copy import deepcopy
 from hashlib import sha1
+from html import unescape
 from pathlib import Path
 from typing import Dict, Tuple, List
 import importlib_resources
@@ -290,11 +291,11 @@ def build(self, run_epubcheck: bool, check_only: bool, build_kobo: bool, build_k
 					raise se.InvalidCssException(f"Couldn’t parse CSS in or near this line: [css]{selector}[/]. Exception: {ex}")
 
 				# We've already replaced attribute/namespace selectors with classes in the CSS, now add those classes to the matching elements
-				if "[epub|type" in selector:
-					for namespace_selector in regex.findall(r"\[epub\|type\~\=\"[^\"]*?\"\]", selector):
+				if regex.search(r"\[[a-z]+\|[a-z]+", selector):
+					for namespace_selector in regex.findall(r"\[[a-z]+\|[a-z]+(?:\~\=\"[^\"]*?\")?\]", selector):
+						new_class = regex.sub(r"^\.", "", se.formatting.namespace_to_class(namespace_selector))
 
 						for element in dom.css_select(namespace_selector):
-							new_class = regex.sub(r"^\.", "", se.formatting.namespace_to_class(namespace_selector))
 							current_class = element.get_attr("class") or ""
 
 							if new_class not in current_class:
@@ -621,6 +622,10 @@ def build(self, run_epubcheck: bool, check_only: bool, build_kobo: bool, build_k
 					# for node in dom.xpath("/html/body//span[contains(@class, 'quote-align')]"):
 					# 	node.unwrap()
 
+					# Change 'noteref' to 'endnote' so that popup endnotes work in Kobo. Kobo doesn't understand 'noteref', only 'endnote'.
+					for node in dom.xpath("/html/body//a[contains(@epub:type, 'noteref')]"):
+						node.set_attr("epub:type", node.get_attr("epub:type") + " endnote")
+
 					# Now add the kobo spans
 					kobo.add_kobo_spans_to_node(dom.xpath("/html/body")[0].lxml_element)
 
@@ -988,6 +993,53 @@ def build(self, run_epubcheck: bool, check_only: bool, build_kobo: bool, build_k
 
 						raise se.BuildFailedException("[bash]epubcheck[/] failed.", build_messages)
 
+			# Now run the Nu Validator
+			with importlib_resources.path("se.data.vnu", "vnu.jar") as jar_path:
+				# We have to use a temp file to hold stdout, because if the output is too large for the output buffer in subprocess.run() (and thus popen()) it will be truncated
+				with tempfile.TemporaryFile() as stdout:
+					subprocess.run(["java", "-jar", str(jar_path), "--format", "xml", str(self.epub_root_path / "epub" / "text")], stdout=stdout, stderr=stdout, check=False)
+
+					stdout.seek(0)
+					vnu_dom = se.easy_xml.EasyXmlTree(stdout.read().decode().strip())
+
+					# The Nu Validator will return errors for epub-specific attributes (like `epub:prefix` and `epub:type`) because they're
+					# not defined in the XHTML5 spec. So, we simply filter out those errors.
+					# We also filter out "section lacks heading" messages, because they are warnings and we may have sections without headings (like dedications, frontispieces)
+					# Also filter out warnings about potentially bad values for datetimes, which are raised for years < 1000. This can occur in works like Omega by Camille Flammarion.
+					messages = vnu_dom.xpath("/messages/*[not(re:test(./message, '^(Attribute (prefix|type) not allowed|(Section|Article) lacks heading.|Potentially bad value.+datetime)'))]")
+
+					for message in messages:
+						message_text = message.xpath("./message")[0].inner_xml()
+						submessage = None
+
+						# Colorize output
+						message_text = regex.sub(r"([Aa]ttribute) <code>", r"\1 [attr]", message_text)
+						message_text = regex.sub(r"([Ee]lement) <code>(.+?)</code>", r"\1 [xhtml]<\2>[/]", message_text)
+						message_text = message_text.replace("<code>", "[xhtml]")
+						message_text = message_text.replace("</code>", "[/]")
+						message_text = unescape(message_text)
+
+						# Do we have a submessage?
+						extract = message.xpath("./extract")
+
+						if extract:
+							# The extract will contain the offending line plus some lines around it.
+							# The offending line is marked up with <m> so pull it out and discard the surrounding lines.
+							target = extract[0].xpath("./m")
+							if target:
+								submessage = target[0].inner_xml()
+							else:
+								submessage = extract[0].inner_xml()
+
+							submessage = unescape(submessage).strip()
+							submessage = [submessage]
+
+						file_path = Path(regex.sub(r"^file:", "", message.get_attr("url")))
+						build_messages.append(BuildMessage("vnu", "", message_text, file_path, message.get_attr("last-line"), message.get_attr("first-column"), submessage))
+
+					if messages:
+						raise se.BuildFailedException("[bash]vnu[/] failed.", build_messages)
+
 		# Now run Ace
 		if run_ace:
 			# We have to use a temp file to hold stdout, because if the output is too large for the output buffer in subprocess.run() (and thus popen()) it will be truncated
@@ -1024,12 +1076,6 @@ def build(self, run_epubcheck: bool, check_only: bool, build_kobo: bool, build_k
 										if file_assertion["earl:test"]["dct:title"] == "valid-lang" and "lang=\"x-" in file_assertion["earl:result"]["html"]:
 											emit_result = False
 
-										# Ace fails if a <section> doesn't have a clear title; but for recomposability, we must nest <section>s without titles.
-										# Attempt to skip that result here, if it looks like we're looking at <section>s nested for recomposability.
-										# See https://standardebooks.org/ebooks/fyodor-dostoevsky/the-brothers-karamazov/constance-garnett
-										if file_assertion["earl:test"]["dct:title"] == "landmark-unique" and len(regex.findall(r"<(section|article)", file_assertion["earl:result"]["html"])) >= 2:
-											emit_result = False
-
 										# Ace fails if an <article> doesn't have a `role` that matches its `epub:type`; but epubcheck
 										# will complain if the `role` is not allowed on that element. This mostly affects <article>s.
 										# We choose to satisfy epubcheck first by not including `role` on <article>, then ignoring Ace's complaint.
@@ -1104,10 +1150,10 @@ def build(self, run_epubcheck: bool, check_only: bool, build_kobo: bool, build_k
 				# Remove se:image.color-depth.black-on-transparent, as Calibre removes media queries so this will *always* be invisible
 				for node in dom.xpath("/html/body//img[contains(@class, 'epub-type-se-image-color-depth-black-on-transparent') or contains(@epub:type, 'se:image.color-depth.black-on-transparent')]"):
 					if node.get_attr("class"):
-						node.set_attr("class", node.get_attr("class").replace("epub-type-se-image-color-depth-black-on-transparent", ""))
+						node.set_attr("class", node.get_attr("class").replace("epub-type-se-image-color-depth-black-on-transparent", "").replace("epub-type-se-image-style-realistic", ""))
 
 					if node.get_attr("epub:type"):
-						node.set_attr("epub:type", node.get_attr("epub:type").replace("se:image.color-depth.black-on-transparent", ""))
+						node.set_attr("epub:type", node.get_attr("epub:type").replace("se:image.color-depth.black-on-transparent", "").replace("se:image.style.realistic", ""))
 
 				# If the only element on the page is an absolutely positioned image, Kindle will ignore the file in the reading order.
 				# So, in that case we add a `<div>` with some text content to fool Kindle.
