@@ -111,7 +111,7 @@ class TocItem:
 
 		return out_string
 
-	def landmark_link(self, work_type: str = "fiction", work_title: str = "WORK_TITLE") -> str:
+	def landmark_link(self, work_title: str = "WORK_TITLE") -> str:
 		"""
 		Generates the landmark item (including list item tags) for the ToC item
 
@@ -127,7 +127,7 @@ class TocItem:
 		if self.place == Position.FRONT:
 			out_string = f"<li>\n<a href=\"text/{self.file_link}\" epub:type=\"frontmatter {self.epub_type}\">{self.title}</a>\n</li>\n"
 		if self.place == Position.BODY:
-			out_string = f"<li>\n<a href=\"text/{self.file_link}\" epub:type=\"bodymatter z3998:{work_type}\">{work_title}</a>\n</li>\n"
+			out_string = f"<li>\n<a href=\"text/{self.file_link}\" epub:type=\"bodymatter\">{work_title}</a>\n</li>\n"
 		if self.place == Position.BACK:
 			out_string = f"<li>\n<a href=\"text/{self.file_link}\" epub:type=\"backmatter {self.epub_type}\">{self.title}</a>\n</li>\n"
 
@@ -172,10 +172,16 @@ def add_landmark(dom: EasyXmlTree, textf: str, landmarks: list) -> None:
 	None
 	"""
 
+	# According to the IDPF a11y best practices page: <http://idpf.org/epub/a11y/techniques/#sem-003>:
+	# > it is recommended to include a link to the start of the body matter as well as to any major
+	# > reference sections (e.g., table of contents, endnotes, bibliography, glossary, index).
+	#
+	# So, we only want the start of the text, and (endnotes,glossary,bibliography,loi) in the landmarks.
+
 	epub_type = ""
-	sections = dom.xpath("//body/*[name() = 'section' or name() = 'article']")
+	sections = dom.xpath("//body/*[name() = 'section' or name() = 'article' or name() = 'nav']")
 	if not sections:
-		raise se.InvalidInputException("Couldn’t locate first [xhtml]<section>[/] or [xhtml]<article>[/].")
+		raise se.InvalidInputException("Couldn’t locate first [xhtml]<section>[/], [xhtml]<article>[/], or [xhtml]<nav>[/].")
 	epub_type = sections[0].get_attr("epub:type")
 	bodys = dom.xpath("//body")
 	if not bodys:
@@ -189,6 +195,12 @@ def add_landmark(dom: EasyXmlTree, textf: str, landmarks: list) -> None:
 	if epub_type in ["frontmatter", "bodymatter", "backmatter"]:
 		return  # if epub_type is ONLY frontmatter, bodymatter, backmatter, we don't want this as a landmark
 
+	if dom.xpath("//*[contains(@epub:type, 'frontmatter')]"):
+		return # We don't want frontmatter in the landmarks
+
+	if dom.xpath("//*[contains(@epub:type, 'backmatter')]") and not regex.findall(r"\b(loi|endnotes|bibliography|glossary|index)\b", epub_type):
+		return # We only want certain backmatter in the landmarks
+
 	# We may wind up with a (front|body|back)matter semantic in epub_type, remove it here since we add it to the landmark later
 	epub_type = regex.sub(r"(front|body|back)matter\s*", "", epub_type)
 
@@ -199,6 +211,9 @@ def add_landmark(dom: EasyXmlTree, textf: str, landmarks: list) -> None:
 		landmark.place = get_place(bodys[0])
 		if epub_type == "halftitlepage":
 			landmark.title = "Half Title"
+		elif epub_type == "titlepage":
+			# Exception: The titlepage always has is titled 'titlepage' in the ToC
+			landmark.title = "Titlepage"
 		else:
 			landmark.title = dom.xpath("//head/title/text()", True)  # Use the page title as the landmark entry title.
 			if landmark.title is None:
@@ -206,7 +221,7 @@ def add_landmark(dom: EasyXmlTree, textf: str, landmarks: list) -> None:
 				landmark.title = landmark.epub_type.capitalize()
 		landmarks.append(landmark)
 
-def process_landmarks(landmarks_list: list, work_type: str, work_title: str) -> str:
+def process_landmarks(landmarks_list: list, work_title: str) -> str:
 	"""
 	Runs through all found landmark items and writes them to the toc file.
 
@@ -232,7 +247,7 @@ def process_landmarks(landmarks_list: list, work_type: str, work_title: str) ->
 	for item in front_items:
 		out_string += item.landmark_link()
 	if body_items:
-		out_string += body_items[0].landmark_link(work_type, work_title)  # Just the first bodymatter item.
+		out_string += body_items[0].landmark_link(work_title)  # Just the first bodymatter item.
 	for item in back_items:
 		out_string += item.landmark_link()
 	return out_string
@@ -284,7 +299,7 @@ def process_items(item_list: list) -> str:
 
 
 
-def output_toc(item_list: list, landmark_list, toc_path: str, work_type: str, work_title: str) -> str:
+def output_toc(item_list: list, landmark_list, toc_path: str, work_title: str) -> str:
 	"""
 	Outputs the contructed ToC based on the lists of items and landmarks found,
 	either to stdout or overwriting the existing ToC file
@@ -329,7 +344,7 @@ def output_toc(item_list: list, landmark_list, toc_path: str, work_type: str, wo
 	navs[1].append(landmark_ol)
 	xhtml = toc_dom.to_string()
 	xhtml = xhtml.replace("TOC_ITEMS", process_items(item_list))
-	xhtml = xhtml.replace("LANDMARK_ITEMS", process_landmarks(landmark_list, work_type, work_title))
+	xhtml = xhtml.replace("LANDMARK_ITEMS", process_landmarks(landmark_list, work_title))
 
 	return se.formatting.format_xhtml(xhtml)
 
@@ -365,6 +380,7 @@ def extract_strings(node: EasyXmlElement) -> str:
 
 	out_string = node.inner_xml()
 	out_string = strip_notes(out_string)
+	out_string = out_string.replace("\n", "")
 	return regex.sub(r"[\n\t]", "", out_string)
 
 def process_headings(dom: EasyXmlTree, textf: str, toc_list: list, nest_under_halftitle: bool, single_file: bool) -> None:
@@ -406,17 +422,12 @@ def process_headings(dom: EasyXmlTree, textf: str, toc_list: list, nest_under_ha
 		# We don't have a heading, so get first content item
 		content_item = dom.xpath("//p | //header | //img")
 		if content_item is not None:
-			parents = content_item[0].xpath("./ancestor::*[name() = 'section' or name() = 'article']")
-			if parents:
-				special_item.level = len(parents)
-			else:  # couldn't find a suitable parent, so just put it at top level
-				special_item.level = 1
-		if nest_under_halftitle:
-			special_item.level += 1
+			special_item.level = get_level(content_item[0], toc_list)
 		special_item.title = dom.xpath("//head/title/text()", True)  # Use the page title as the ToC entry title.
 		if special_item.title is None:
 			special_item.title = "NO TITLE"
 		special_item.file_link = textf
+		special_item.place = place
 		toc_list.append(special_item)
 		return
 
@@ -431,15 +442,47 @@ def process_headings(dom: EasyXmlTree, textf: str, toc_list: list, nest_under_ha
 			# if it's not a bodymatter item we don't care about whether it's single_file
 			toc_item = process_a_heading(heading, textf, is_toplevel, False)
 
-		# Tricky check to see if we want to include the item because there's a halftitle
-		# but only a single content file with no subsidiary sections.
-		if is_toplevel and single_file and nest_under_halftitle and len(heads) == 1:
-			continue
-		if nest_under_halftitle:
-			toc_item.level += 1
+		toc_item.level = get_level(heading, toc_list)
+		toc_item.place = place
+
+		# Exception: The titlepage always has is titled 'titlepage' in the ToC
+		if dom.xpath("//section[re:test(@epub:type, '\\btitlepage\\b')]"):
+			toc_item.title = "Titlepage"
+
 		is_toplevel = False
 		toc_list.append(toc_item)
 
+
+def get_level(node: EasyXmlElement, toc_list: list) -> int:
+	"""
+	Get level of a node.
+	"""
+
+	# first need to check how deep this heading is within the current file
+	parent_sections = node.xpath("./ancestor::*[name() = 'section' or name() = 'article']")
+	if parent_sections:
+		depth = len(parent_sections)
+	else:
+		depth = 1
+
+	if not node.parent:
+		return depth  # must be at the top level
+
+	data_parents = node.xpath("//*[@data-parent]")
+	if not data_parents:
+		return depth
+
+	data_parent = data_parents[0].get_attr("data-parent")
+
+	if data_parent:
+		# see if we can find it in already processed (as we should if spine is correctly ordered)
+		parent_file = [t for t in toc_list if t.id == data_parent]
+		if parent_file:
+			this_level = parent_file[0].level + 1
+			return this_level + depth - 1  # subtract from depth because all headings should have depth >= 1
+
+	return depth
+
 def process_a_heading(node: EasyXmlElement, textf: str, is_toplevel: bool, single_file: bool) -> TocItem:
 	"""
 	Generate and return a single TocItem from this heading.
@@ -455,11 +498,6 @@ def process_a_heading(node: EasyXmlElement, textf: str, is_toplevel: bool, singl
 	"""
 
 	toc_item = TocItem()
-	parent_sections = node.xpath("./ancestor::*[name() = 'section' or name() = 'article']")
-	if parent_sections:
-		toc_item.level = len(parent_sections)
-	else:
-		toc_item.level = 1
 
 	toc_item.division = get_book_division(node)
 
@@ -561,6 +599,8 @@ def evaluate_descendants(node: EasyXmlElement, toc_item: TocItem) -> TocItem:
 				child_strings = regex.sub(r"\bordinal\b", "", child_strings)
 				# remove extra spaces
 				child_strings = regex.sub(r"[ ]{2,}", " ", child_strings)
+				# remove any carriage returns
+				child_strings = regex.sub(r"\n", "", child_strings)
 				# get rid of any endnotes
 				child_strings = strip_notes(child_strings)
 				toc_item.title = child_strings.strip()
@@ -670,21 +710,27 @@ def process_all_content(file_list: list) -> Tuple[list, list]:
 	single_file = (len(body_items) == 1)
 
 	nest_under_halftitle = False
-	place = Position.NONE
+
 	for textf in file_list:
 		with open(textf, "r", encoding="utf-8") as file:
 			dom = se.easy_xml.EasyXmlTree(file.read())
-		body = dom.xpath("//body")
-		if body:
-			place = get_place(body[0])
-		else:
-			raise se.InvalidInputException("Couldn’t locate [xhtml]<body>[/].")
-		if place == Position.BACK:
-			nest_under_halftitle = False
 		process_headings(dom, textf.name, toc_list, nest_under_halftitle, single_file)
 		if dom.xpath("/html/body//*[contains(@epub:type, 'halftitlepage')]"):
 			nest_under_halftitle = True
 
+	# now go through adjusting for nesting under halftitle
+	if nest_under_halftitle:
+		# tricky because a few books have forewords, etc AFTER the halftitle, so have to know if we've passed it
+		passed_halftitle = False
+		for toc_item in toc_list:
+			if toc_item.place == Position.BODY:
+				toc_item.level += 1
+			if passed_halftitle and toc_item.place == Position.FRONT:
+				toc_item.level += 1
+			if "halftitle" in toc_item.file_link:
+				passed_halftitle = True
+
+
 	# We add this dummy item because outputtoc always needs to look ahead to the next item.
 	last_toc = TocItem()
 	last_toc.level = 1
@@ -699,8 +745,7 @@ def generate_toc(self) -> str:
 	"""
 
 	work_title = self.get_work_title()
-	work_type = self.get_work_type()
 
 	landmarks, toc_list = process_all_content(self.spine_file_paths)
 
-	return output_toc(toc_list, landmarks, self.toc_path, work_type, work_title)
+	return output_toc(toc_list, landmarks, self.toc_path, work_title)
